# A mail collector thingy using pop3
import gtk
import timeoutsocket
import poplib
import string
from boxtypes.net_exceptions import *
from ptk.big_edit_box import *
import pynei18n

# Authentication methods
AUTH_POP = 0
AUTH_APOP = 1
AUTH_RPOP = 2

auth_names = [ "POP", "APOP", "RPOP" ]

class pop3_recv:
	def __init__(self, conf):
		self.conf = conf
	
	def check_renew_connection(self):
		if not self.__dict__.has_key("connection"):
			self.get_connection()
	
	def get_connection(self):
		self.connection = poplib.POP3(self.conf["server"], self.conf["port"])
		self.auth()
	
	def close_connection(self):
		if self.__dict__.has_key("connection"):
			self.connection.quit()

	def auth(self):
		try:
			if self.conf["auth"] == AUTH_POP:
				if self.conf["username"] != "":
					self.connection.user(self.conf["username"])
				if self.conf["password"] != "":
					self.connection.pass_(self.conf["password"])
			elif self.conf["auth"] == AUTH_APOP:
				self.connection.apop(self.conf["username"], self.conf["password"])
			elif self.conf["auth"] == AUTH_RPOP:
				self.connection.rpop(self.conf["username"])
		except poplib.error_proto, e:
			raise auth_exception(e)

	def get_msg_list(self):
		"""
		Return dict of message ids in form:
		{ 'msg1_id': msg1_size, 'msg2_id': msg2_size, ... }
		"""
		raw_list = self.connection.list()[1]
		y = {}
		for i in raw_list:
			id, size = string.split(i)
			y[id] = int(size)
		return y

	def get_header(self, index):
		lines = self.connection.top(index, 0)[1]
		head = string.join(lines, "\n")
		head = string.replace(head, "\000", "")
		return head

	def get_body(self, index):
		lines = self.connection.retr(index)[1]
		body = string.join(lines, "\n")
		start = string.find(body, "\n\n") + 2
		body = string.replace(body, "\000", "")
		return body[start:]

	def delete_msg(self, index):
		self.connection.dele(index)

def fix_defaults(conf):
	conf["port"] = 110
	conf["auth"] = AUTH_POP

def get_connection(conf, no_connection=0):
	"""
	return a connection object thingy.
	"""
	c = pop3_recv(conf)
	if not no_connection:
		c.get_connection()
	return c

def setup(parent_mailbox, conf, parent_win):
	"""
	Open a configuration dialog box.
	"""
	win = gtk.Window()
	win.set_title("%s POP3 Settings" % parent_mailbox.name)
	win.set_transient_for(parent_win)

	box = gtk.VBox()
	box.set_border_width(5)
	win.add(box)
	box.show()

	# dummy class object for settings
	class _dummy:
		pass
	settings = _dummy()
	settings.__dict__ = conf

	settings_box0 = big_edit_box( settings,
	      ( ("server", _("POP3 server:"), VAR_TYPE_STRING, 0, 0),
		("port", _("Port:"), VAR_TYPE_INTEGER, 0, 110),
		("username", _("Username:"), VAR_TYPE_STRING, 0, 0),
		("password", _("Password:"), VAR_TYPE_STRING, MOD_HIDE, 0) )
	)
	box.pack_start(settings_box0, expand=False)
	settings_box0.show()

	auth_method = [ conf["auth"] ]

	def _ch_auth(item, auth_method=auth_method):
		auth_method[0] = auth_combo.get_active ()

	# authentication method
	auth_combo = gtk.combo_box_new_text ()
	for i in auth_names:
		auth_combo.append_text (i)
	auth_combo.set_active (conf["auth"])
	auth_combo.connect ("changed", _ch_auth)

	hbox = gtk.HBox(spacing=5)
	hbox.set_border_width(5)
	box.pack_start(hbox)

	label = gtk.Label(_("Authentication:"))
	hbox.pack_start(label, expand=False)
	label.show()
	
	hbox.pack_start(auth_combo, expand=False)
	auth_combo.show()
	hbox.show()

	# separator between entry boxes and buttons
	separator = gtk.HSeparator()
	box.pack_start(separator, expand=False)
	separator.show()

	def save_changes(_button, win=win, settings=settings, settings_box0=settings_box0, \
			auth_method=auth_method, conf=conf):
		# Extract info
		settings_box0.apply_changes()
		conf["auth"] = auth_method[0]
		win.destroy()

	def _cancel(_button, win=win):
		win.destroy()

	# boootons
	buttonbox = gtk.HBox(spacing=5)
	buttonbox.set_border_width(5)
	box.pack_start(buttonbox, expand=False)
	buttonbox.show()

	button = gtk.Button(stock="gtk-cancel")
	button.connect("clicked", _cancel)
	buttonbox.pack_end(button, expand=False)
	button.show()

	button = gtk.Button(stock="gtk-ok")
	button.connect("clicked", save_changes)
	buttonbox.pack_end(button, expand=False)
	button.show()

	win.show()

